from autopost.models import AutoTask, CrawlLogs
from urllib import request
from bs4 import BeautifulSoup, Tag
import json, shortuuid
from autopost.utils.html import HtmlStorage
from autopost.models import Article, Category
from django.conf import settings
from uuslug import uuslug


class QtCrawler:
    """
    简易爬虫代码
    """

    task = None

    def crawl_news(self, task_id):
        try:
            self.task = AutoTask.objects.get(pk=task_id)
        except AutoTask.DoesNotExist:
            return False
        # 根据规则找出文章列表
        self.crawl_list(self.task.url, self.task.list_selector)
        return True

    @staticmethod
    def get_content(url):
        """
        封装url请求
        :param url:
        :return: Beautifulsoup content
        """
        user_agent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_0) AppleWebKit/537.36 (KHTML, like Gecko) ' \
                     'Chrome/69.0.3497.100 Safari/537.36 '
        headers = {'User-Agent': user_agent}
        req = request.Request(url, headers=headers)
        resp = request.urlopen(req)
        bs_content = BeautifulSoup(resp, 'lxml')
        return bs_content, req

    def crawl_list(self, url, pattern, preview=False):
        html, req = self.get_content(url)
        articles = html.select(pattern)

        ret_data = list()
        for article in articles:
            if 'href' not in article.attrs:
                continue
            item_url = article['href']
            item_title = article.string

            # 完善URL
            url_base = req.full_url.replace(req.selector, '')
            if str(item_url).startswith('//'):
                item_url = req.type + ':' + item_url
            if not str(item_url).startswith('http') and not str(item_url).startswith('//'):
                item_url = url_base + item_url
            if not preview:
                # 抓取文章详情
                try:
                    title, html_content, first_img, key_words, summery = self.crawl_detail(item_url,
                                                                                           self.task.title_selector,
                                                                                           self.task.content_selector,
                                                                                           self.task.content_except,
                                                                                           self.task.title_replace,
                                                                                           self.task.content_replace,
                                                                                           self.task.thumbnail_selector,
                                                                                           lazy_photo=self.task.lazy_photo,
                                                                                           )
                    if not Article.objects.filter(title=title).exists():
                        self.save_article(title, html_content, first_img, key_words, summery)

                except ChildProcessError:
                    continue
            ret_data.append({'title': item_title, 'url': item_url})
        return ret_data

    def crawl_detail(self, url, title_pattern, content_pattern, content_exclude, title_replace, content_replace,
                     thumbnail_pattern='', lazy_photo='', fetch_img=True,
                     preview=False):
        """
        抓取文章详情
        :param url:
        :param title_pattern:
        :param content_pattern:
        :param content_exclude:
        :param title_replace:
        :param content_replace:
        :param thumbnail_pattern:
        :param lazy_photo:
        :param fetch_img:
        :param preview:
        :return:
        """
        html, req = self.get_content(url)
        try:
            title = html.select_one(title_pattern).text.strip()
            # 检查文章是否被抓取过 不同站点可以重复抓取
            if not preview and CrawlLogs.objects.filter(title=title, site=self.task.site).exists():
                # 直接返回文章
                db_article = Article.objects.filter(title=title).get()
                return db_article.title, db_article.content, db_article.thumbnail, db_article.keywords, db_article.summery
            contents_list = html.select(content_pattern)
            main_html = ''
            for tags in contents_list:
                main_html += tags.prettify()
            main_content = BeautifulSoup(main_html)
        except Exception:
            raise ChildProcessError
        try:
            exclude_obj = json.loads(content_exclude)
            if 'script' not in exclude_obj:
                exclude_obj.append('script')
        except Exception:
            exclude_obj = ['script']
        for item in exclude_obj:
            try:
                main_content.select_one(item).replace_with('')
            except Exception:
                continue
        try:
            title_rule = eval(title_replace)
            for item in title_rule:
                title = str(title).replace(item[0], item[1])
        except Exception:
            print('Do not need replace title.')

        # 获取缩略图
        thumbnail_url = ''
        if thumbnail_pattern:
            image_obj = html.select_one(thumbnail_pattern)
            if isinstance(image_obj, Tag):
                img_src = image_obj.attrs['src']
                img_name = shortuuid.uuid()
                ret, info = HtmlStorage.fetch_qiniu(img_src, img_name)
                thumbnail_url = settings.QINIU_URL + ret['key']

        # 获取图片上传
        html_content, first_img, key_words, summery = HtmlStorage.save_content(main_content, send_qiniu=True,
                                                                               save_img=fetch_img,
                                                                               lazy_photo=lazy_photo,
                                                                               thumbnail=thumbnail_url)
        try:
            content_rule = eval(content_replace)
            for item in content_rule:
                html_content = html_content.replace(item[0], item[1])
        except Exception as e:
            print('Do not need replace content. %s' % str(e))
        if preview:
            return title, html_content
        else:
            url_base = req.full_url.replace(req.selector, '')
            crawl_log = CrawlLogs()
            crawl_log.title = title
            crawl_log.url = url
            crawl_log.site = self.task.site
            crawl_log.site_url = url_base
            crawl_log.save()
        return title, html_content, first_img, key_words, summery

    def save_article(self, title, html_content, first_img, key_words, summery):
        """
        保存文章
        :param title:
        :param html_content:
        :param first_img:
        :param key_words:
        :param summery:
        :return:
        """
        article = Article()
        article.site = self.task.site
        article.category = self.task.category
        article.title = title
        article.content = html_content
        article.keywords = key_words
        article.summery = summery
        article.thumbnail = first_img
        article.author = self.task.author
        article.slug = uuslug(title, article, max_length=25, word_boundary=True)
        article.save()
        return article



# Generated by Django 2.1.2 on 2019-09-03 01:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('autopost', '0008_auto_20190829_0935'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='slug',
            field=models.CharField(blank=True, max_length=128, verbose_name='标识'),
        ),
        migrations.AlterField(
            model_name='category',
            name='status',
            field=models.IntegerField(choices=[(1, '显示'), (0, '隐藏'), (2, '主菜单')], default=1, verbose_name='状态'),
        ),
    ]



# Generated by Django 2.1.2 on 2019-09-06 15:18

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('autopost', '0009_auto_20190903_0948'),
    ]

    operations = [
        migrations.AlterField(
            model_name='article',
            name='category',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='category', to='autopost.Category', verbose_name='所属分类'),
        ),
        migrations.AlterField(
            model_name='article',
            name='site',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.DO_NOTHING, to='sites.Site', verbose_name='所属站点'),
        ),
        migrations.AlterField(
            model_name='category',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='children', to='autopost.Category', verbose_name='上级菜单'),
        ),
        migrations.AlterField(
            model_name='category',
            name='status',
            field=models.IntegerField(choices=[(1, '显示'), (2, '主菜单'), (0, '隐藏')], default=1, verbose_name='状态'),
        ),
    ]



from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from simditor.fields import RichTextField
from mptt.models import MPTTModel, TreeForeignKey


# 通用评论
class Comment(MPTTModel):
    title = models.CharField('标题', max_length=128, blank=True, null=True)
    content = RichTextField()
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        related_name='comments'
    )

    # mptt树形结构
    parent = TreeForeignKey(
        'self',
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        related_name='children'
    )

    '''
    记录二级评论回复给谁
    限制评论最多只能两级:
    一级评论人为 a，二级评论人为 b（parent 为 a），三级评论人为 c（parent 为 b）。
    因为我们不允许评论超过两级，因此将 c 的 parent 重置为 a，reply_to 记录为 b，这样就能正确追溯真正的被评论者了
    '''
    reply_to = models.ForeignKey(
        User,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        related_name='repliers'
    )

    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        db_table = 'comments'

    class MPTTMeta:
        order_insertion_by = ['created']

    def __str__(self):
        return self.title



# Generated by Django 2.1.2 on 2019-08-25 09:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system', '0007_auto_20190731_1528'),
    ]

    operations = [
        migrations.AddField(
            model_name='recommended',
            name='slug',
            field=models.SlugField(blank=True, verbose_name='标签'),
        ),
        migrations.AlterField(
            model_name='menu',
            name='type',
            field=models.IntegerField(choices=[(3, '论坛'), (2, '文章'), (1, '店铺')], default=1, verbose_name='类别'),
        ),
    ]



from django.urls import path
from . import views

app_name = 'comment'

urlpatterns = [
    # 发表评论
    path('post-comment/<int:article_id>', views.post_comment, name='post_comment'),
    path('post-comment/<int:article_id>/<int:parent_comment_id>', views.post_comment, name='comment_reply')
]



# Generated by Django 2.1.2 on 2019-05-07 15:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system', '0005_auto_20181108_0915'),
    ]

    operations = [
        migrations.AlterField(
            model_name='menu',
            name='url',
            field=models.CharField(blank=True, max_length=255, verbose_name='地址'),
        ),
    ]



# Generated by Django 2.1.2 on 2019-08-29 01:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('autopost', '0007_auto_20190507_2329'),
    ]

    operations = [
        migrations.AddField(
            model_name='autotask',
            name='thumbnail_selector',
            field=models.CharField(blank=True, max_length=128, verbose_name='缩略图规则'),
        ),
        migrations.AlterField(
            model_name='article',
            name='status',
            field=models.BooleanField(choices=[(True, '是'), (False, '否')], default=True, verbose_name='是否显示'),
        ),
        migrations.AlterField(
            model_name='category',
            name='status',
            field=models.IntegerField(choices=[(1, '显示'), (2, '主菜单'), (0, '隐藏')], default=1, verbose_name='状态'),
        ),
        migrations.AlterField(
            model_name='remoteimages',
            name='origin_src',
            field=models.CharField(db_index=True, max_length=255, verbose_name='原始地址'),
        ),
        migrations.AlterField(
            model_name='remoteimages',
            name='remote_src',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='远程地址'),
        ),
    ]



from django.shortcuts import render, get_object_or_404, redirect
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, JsonResponse

from .forms import CommentForm
from article.models import ArticlePost
from .models import Comment

from notifications.signals import notify
from django.contrib.auth.models import User


# 文章评论
@login_required(login_url='/userprofile/login/')
def post_comment(request, article_id, parent_comment_id=None):
    article = get_object_or_404(ArticlePost, id=article_id)

    # 处理 POST 请求
    if request.method == 'POST':
        comment_form = CommentForm(request.POST)
        if comment_form.is_valid():
            new_comment = comment_form.save(commit=False)
            new_comment.article = article
            new_comment.user = request.user

            # 二级回复
            if parent_comment_id:
                parent_comment = Comment.objects.get(id=parent_comment_id)
                # 若回复层级超过二级，则转换为二级
                new_comment.parent_id = parent_comment.get_root().id
                # 被回复人
                new_comment.reply_to = parent_comment.user
                new_comment.save()

                # 给其他用户发送通知
                if not parent_comment.user.is_superuser and not parent_comment.user == request.user:
                    notify.send(
                        request.user,
                        recipient=parent_comment.user,
                        verb='回复了你',
                        target=article,
                        action_object=new_comment,
                    )

                # return HttpResponse("200 OK")
                return JsonResponse({"code": "200 OK", "new_comment_id": new_comment.id})

            new_comment.save()
            
            # 给管理员发送通知
            if not request.user.is_superuser:
                notify.send(
                        request.user,
                        recipient=User.objects.filter(is_superuser=1),
                        verb='回复了你',
                        target=article,
                        action_object=new_comment,
                    )

            # 添加锚点
            redirect_url = article.get_absolute_url() + '#comment_elem_' + str(new_comment.id)
            return redirect(redirect_url)
        else:
            return HttpResponse("表单内容有误，请重新填写。")
    # 处理 GET 请求
    elif request.method == 'GET':
        comment_form = CommentForm()
        context = {
            'comment_form': comment_form,
            'article_id': article_id,
            'parent_comment_id': parent_comment_id
        }
        return render(request, 'comment/reply.html', context)
    # 处理其他请求
    else:
        return HttpResponse("仅接受GET/POST请求。")



"""django_module URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, include, re_path
from django.views.static import serve
from django.conf import settings
from django.views.i18n import JavaScriptCatalog

urlpatterns = [
    path('grappelli/', include('grappelli.urls')),
    path('admin/', admin.site.urls),
    re_path(r'^chaining/', include('smart_selects.urls')),
    re_path(r'^simditor/', include('simditor.urls')),
    path('autopost/', include('autopost.urls')),
    # path('uc/', include('usercenter.urls'), name='uc'),
    re_path(r'^jsi18n/$', JavaScriptCatalog.as_view(), name='javascript-catalog'),
]

# 生产环境使用nginx路由
if settings.ENV_CONFIG == 'dev':
    urlpatterns.append(re_path(r'^media/(?P<path>.*)$', serve, {'document_root': settings.MEDIA_ROOT}))



# Generated by Django 2.1.2 on 2019-07-31 07:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system', '0006_auto_20190507_2329'),
    ]

    operations = [
        migrations.CreateModel(
            name='Recommended',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(db_index=True, max_length=128, verbose_name='标签')),
                ('status', models.IntegerField(choices=[(2, '隐藏'), (1, '显示')], default=1, verbose_name='状态')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
            ],
            options={
                'verbose_name': '推荐标签',
                'verbose_name_plural': '推荐标签管理',
                'db_table': 'sys_recommend',
            },
        ),
        migrations.AlterField(
            model_name='menu',
            name='status',
            field=models.IntegerField(choices=[(2, '隐藏'), (1, '显示')], default=1, verbose_name='状态'),
        ),
        migrations.AlterField(
            model_name='menu',
            name='type',
            field=models.IntegerField(choices=[(1, '店铺'), (3, '论坛'), (2, '文章')], default=1, verbose_name='类别'),
        ),
    ]



# Generated by Django 2.1.2 on 2019-09-06 15:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system', '0009_auto_20190831_0726'),
    ]

    operations = [
        migrations.AlterField(
            model_name='menu',
            name='status',
            field=models.IntegerField(choices=[(1, '显示'), (2, '隐藏')], default=1, verbose_name='状态'),
        ),
        migrations.AlterField(
            model_name='menu',
            name='type',
            field=models.IntegerField(choices=[(2, '文章'), (1, '店铺'), (3, '论坛')], default=1, verbose_name='类别'),
        ),
        migrations.AlterField(
            model_name='recommended',
            name='status',
            field=models.IntegerField(choices=[(1, '显示'), (2, '隐藏')], default=1, verbose_name='状态'),
        ),
        migrations.AlterUniqueTogether(
            name='menu',
            unique_together={('parent', 'slug')},
        ),
    ]
